function drawLine(pt1, pt2, meter)
	--Line drawing function created by FreezingClouds
	local meterLineOut = SKIN:GetMeter(meter)
	local TLPt = {pt1[1] < pt2[1] and pt1[1] or pt2[1],pt1[2] < pt2[2] and pt1[2] or pt2[2]}
	meterLineOut:SetX(TLPt[1])
	meterLineOut:SetY(TLPt[2])
	meterLineOut:SetW(math.abs(pt2[1]-pt1[1]))
	meterLineOut:SetH(math.abs(pt2[2]-pt1[2]))
	local angle = math.atan((pt2[2]-pt1[2])/(pt2[1]-pt1[1]))
	SKIN:Bang('!SetOption '..meter..' StartAngle '..tostring(angle))
	local distance = math.sqrt(((pt2[1]-pt1[1])^2)+((pt2[2]-pt1[2])^2))
	SKIN:Bang('!SetOption '..meter..' LineStart '..tostring(-distance/2))
	SKIN:Bang('!SetOption '..meter..' LineLength '..tostring(distance/2))
end
function deg2x(degree, center, radius)
	return math.cos(math.rad(degree))*radius+center[1]
end
function deg2y(degree, center, radius)
	return math.sin(math.rad(degree))*radius+center[2]
end
function clamp(x, min, max)
	return (x > min and (x < max and x or max) or min)
end
function findVal(str, valnum)
	str = str..','
	local slice = str
	for i = 1, valnum do
		slice = string.sub(slice,string.find(slice,',')+1,-1)
	end
	return tonumber(string.sub(slice,0,string.find(slice,',')-1))
end
function Initialize()
	file = io.open(SKIN:MakePathAbsolute(SELF:GetOption('OutFile')),'w')
	count = tonumber(SKIN:GetVariable('Bars'))
	if count < 2 then
		count = 2
	elseif count > 360 then
		count = 360
	end
	height = tonumber(SKIN:GetVariable('BarHeight'))
	width = tonumber(SKIN:GetVariable('BarWidth'))
	radius = tonumber(SKIN:GetVariable('CoverSize'))/2
	degInterval = 360/count
	center = {height+tonumber(SKIN:GetVariable('CoverSize')),height+tonumber(SKIN:GetVariable('CoverSize'))}
	rotoff = 0
	
	local innerColor = SKIN:GetVariable('InnerGradientColor')
	red1 = findVal(innerColor,0)
	green1 = findVal(innerColor,1)
	blue1 = findVal(innerColor,2)
	local outerColor = SKIN:GetVariable('OuterGradientColor')
	local red2 = findVal(outerColor,0)
	local green2 = findVal(outerColor,1)
	local blue2 = findVal(outerColor,2)
	
	diffRed = red2 - red1
	diffGreen = green2 - green1
	diffBlue = blue2 - blue1

	averageSize = tonumber(SKIN:GetVariable('AverageSize'))
	amp = tonumber(SKIN:GetVariable('Amplifier'))
	EqlArr63 = {5,4.5,4,3,2,1.5,1.25,1.0,1.0,1.25,1.25,1.5,1.5,1.25,1.25,1.5,2,2,1.75,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.5,1.75,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2}
	EqlArrX = {}
	for i = 1, count do
		EqlArrX[i] = (EqlArr63[clamp(math.floor((i*63)/count),1,63)]+EqlArr63[clamp(math.ceil((i*63)/count),1,63)])/2
	end
	for i = 1, count do
		file:write('[AudioProcessor',i,']\n')
		file:write('Measure=Plugin\n')
		file:write('Plugin=AudioLevel\n')
		file:write('Parent=AudioExtractor\n')
		file:write('Type=Band\n')
		file:write('BandIdx=',i-1,'\n')
		file:write('Channel=Avg\n')
		if ((-tonumber(SKIN:GetVariable('Equalizer'))+1)+tonumber(SKIN:GetVariable('DoubleSmoothing')) ~= 0) then
			file:write('AverageSize=',averageSize,'\n')
		end
	end
	measureType = 'AudioProcessor'
	if (tonumber(SKIN:GetVariable('Equalizer')) == 1) then
		file:write('[Equalizer1]\n')
		file:write('Measure=Calc\n')
		file:write('Formula=Clamp(((AudioProcessor1*2+AudioProcessor2)/',EqlArrX[1],')*',amp,',0,1)\n')
		file:write('AverageSize=',averageSize,'\n')
		for i = 2, count-1 do
			file:write('[Equalizer',i,']\n')
			file:write('Measure=Calc\n')
			file:write('Formula=Clamp(((AudioProcessor',i-1,'+AudioProcessor',i,'+AudioProcessor',i+1,')/',EqlArrX[i],')*',amp,',0,1)\n')
			file:write('AverageSize=',averageSize,'\n')
		end
		file:write('[Equalizer',count,']\n')
		file:write('Measure=Calc\n')
		file:write('Formula=Clamp(((AudioProcessor',count-1,'+AudioProcessor',count,'*2)/',EqlArrX[count],')*',amp,',0,1)\n')
		file:write('AverageSize=',averageSize,'\n')
		measureType = 'Equalizer'
	end
	for i = 1, count do
		file:write('[MeterLine',i,']\n')
		file:write('Meter=Roundline\n')
		file:write('MeterStyle=MSLine\n')
	end
	if (tonumber(SKIN:GetVariable('TipRounding')) == 1) then
		for i = 1, count do
			file:write('[MeterTip',i,']\n')
			file:write('Meter=Roundline\n')
			file:write('MeterStyle=MSTip\n')
		end
	end
	file:close()
end
function Update()
	local rotspd = clamp(SKIN:GetMeasure('AudioProcessor1'):GetValue()*2-1,0.1,1)
	for i = 1,count do
		local value = SKIN:GetMeasure(measureType..i):GetValue()
		local point = {deg2x(degInterval*i+rotoff, center, radius+height*value*0.95+2)+1,deg2y(degInterval*i+rotoff, center, radius+height*value*0.95+2)+1}
		drawLine({deg2x(degInterval*i+rotoff,center,radius)+1,deg2y(degInterval*i+rotoff,center,radius)+1},point,'MeterLine'..i)
		if (tonumber(SKIN:GetVariable('TipRounding')) == 1) then
			local meter = SKIN:GetMeter('MeterTip'..i)
			meter:SetX(deg2x(degInterval*i+rotoff, center, radius+height*value*0.95+2)+1)
			meter:SetY(deg2y(degInterval*i+rotoff, center, radius+height*value*0.95+2)+1)
			SKIN:Bang('!SetOption MeterTip'..i..' LineColor '..tostring(red1+diffRed*value)..','..tostring(green1+diffGreen*value)..','..tostring(blue1+diffBlue*value))
		end
		SKIN:Bang('!SetOption MeterLine'..i..' LineColor '..tostring(red1+diffRed*value)..','..tostring(green1+diffGreen*value)..','..tostring(blue1+diffBlue*value))
	end
	if (rotoff+rotspd > 360) then
		rotoff = rotoff+rotspd-360
	else
		rotoff = rotoff+rotspd
	end
end